package video.pano.rtc.flutter

import com.pano.rtc.api.PanoAnnotationManager
import com.pano.rtc.api.RtcNetworkManager
import com.pano.rtc.api.RtcVideoStreamManager
import com.pano.rtc.api.RtcWhiteboard
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import video.pano.rtc.PanoRtcPlugin
import video.pano.rtc.native.*
import video.pano.rtc.native.api.IRtcManagerCreator
import kotlin.reflect.full.declaredMemberFunctions
import kotlin.reflect.jvm.javaMethod

class RtcEngineManagerWrapper : FlutterWrapper(), IRtcManagerCreator {

    val manager = RtcEngineManager(this, this@RtcEngineManagerWrapper::emit)
    private val rtcWhiteboardEngineWrapper = RtcWhiteboardEngineWrapper()

    init {
        methodChannel = PanoRtcPlugin.createMethodChannel("pano_rtc/api_engine")
        eventChannel = PanoRtcPlugin.createEventChannel("pano_rtc/events_engine")
        methodChannel.setMethodCallHandler(this)
        eventChannel?.setStreamHandler(this)
    }

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        manager::class.declaredMemberFunctions.find { it.name == call.method }?.let { function ->
            function.javaMethod?.let { method ->
                try {
                    val parameters = mutableListOf<Any?>()
                    call.arguments<Map<*, *>>()?.toMutableMap()?.let {
                        if (call.method == "create") {
                            it["context"] = PanoRtcPlugin.appContext
                        }
                        parameters.add(it)
                    }
                    method.invoke(manager, *parameters.toTypedArray(), ResultCallback(result))
                    return@onMethodCall
                } catch (e: Exception) {
                    e.printStackTrace()
                }
            }
        }
        result.notImplemented()
    }

    override fun createWhiteboardEngine(whiteboardId: String, whiteboard: RtcWhiteboard?): RtcWhiteboardEngine? {
        return rtcWhiteboardEngineWrapper.createWhiteboardEngine(whiteboardId, whiteboard)
    }

    override fun createAnnotationMgr(manager: PanoAnnotationManager?): RtcAnnotationMgr? {
        return RtcAnnotationMgrWrapper(manager).manager
    }

    override fun createNetworkMgr(manager: RtcNetworkManager?): RtcNetworkMgr? {
        return RtcNetworkMgrWrapper(manager).manager
    }

    override fun createVideoStreamMgr(manager: RtcVideoStreamManager?): RtcVideoStreamMgr? {
        return RtcVideoStreamMgrWrapper(manager).manager
    }
}